@@ -26,7 +26,7 @@ def make_policy(args, pc=None, optional=False):
         -m <MINUTES>  Request verification timeouts of this many minutes
         -I            Ignore any default settings
     """
-    from .client import PasscrowIdentityPolicy, PasscrowClientPolicy
+    from .client import PasscrowIdentityPolicy, PasscrowRecoveryPolicy
 
     defaults = not args.get('-I')
     exp_days = int(args.get('-d', [0])[0]) or None
@@ -70,7 +70,7 @@ def make_policy(args, pc=None, optional=False):
     else:
         n = m = None
 
-    return PasscrowClientPolicy(idps=idps, n=n, m=m,
+    return PasscrowRecoveryPolicy(idps=idps, n=n, m=m,
         expiration_days=exp_days,
         timeout_minutes=tmo_mins)
 
@@ -128,10 +128,10 @@ def cli_init(args):
     pc = make_pc(args, create_dirs=True)
 
     # Enable the test server by default
-    pc.default_policy.servers = ['mailto via passcrow-test.mailpile.is']
+    pc.default_policy.servers = ['email via passcrow-test.mailpile.is']
     sys.stderr.write("""\
 NOTE: Configuring `passcrow-test.mailpile.is` as the default server for
-      e-mail (mailto) verification. Note that this server is for testing
+      email (a.k.a. mailto) verification. Note this server is for testing
       only and deletes all data after 30 days, no matter what expiration
       has been requested/promised.\n\n""")
 
@@ -296,12 +296,12 @@ def cli_recover(args):
 
     Example:
         passcrow recover "Secret 1"
-        passcrow recover "Secret 1" A-123423 B-999888 C-333444
+        passcrow recover "Secret 1" 1123423 3999888 2333444
 
     Ephemeral example:
         passcrow recover passcrow.example.org:AbCd-1234-FfIi-Xyz0
-        passcrow recover AbCd-1234-FfIi-Xyz0 A-123123
-        passcrow recover AbCd-1234-FfIi-Xyz0 A-654321 B-123123 C-...
+        passcrow recover AbCd-1234-FfIi-Xyz0 1123123
+        passcrow recover AbCd-1234-FfIi-Xyz0 4654321 1123123 2...
 
     Options:
         -o <FILENAME> Write recovered data to file (default is stdout)
@@ -393,7 +393,7 @@ with the codes as arguments. For example:
 You must provide at least %d code(s) within %d minutes.""" % (
                     hint_list, url_list,
                     pack.name,
-                    ' '.join(['A-123456', 'B-321123'][:pack.min_shares]),
+                    ' '.join(['1234567', '2321123'][:pack.min_shares]),
                     pack.min_shares,
                     round(timeout/60)))
         return True
