@@ -8,7 +8,8 @@ import appdirs
 
 from .proto import *
 
-from .handlers.email import MailtoHandler
+from . import VERSION
+from .handlers.email import EmailHandler
 from .payments import PaymentFree, PaymentHashcash
 from .secret_share import random_int
 from .storage import FileSystemStorage
@@ -41,6 +42,18 @@ class JsonError(_json_object):
     error = property(*_json_object_prop('error'))
 
 
+class ServerStats(_json_object):
+    _KEYS = {
+        'version': str,
+        'start-ts': int,
+        'requests': dict,
+        'storage': dict}
+    version = property(*_json_object_prop('version'))
+    start_ts = property(*_json_object_prop('start-ts'))
+    requests = property(*_json_object_prop('requests'))
+    storage = property(*_json_object_prop('storage'))
+
+
 class PasscrowServer:
     STORAGE_TABLES = {
         'escrow': ['data'],
@@ -51,7 +64,6 @@ class PasscrowServer:
             log=None,
             handlers=None,
             payments=None,
-            warnings_to=None,
             country_code=None,
             about_url=None,
             expiration=None,
@@ -65,6 +77,11 @@ class PasscrowServer:
         self.expiration = expiration or DEFAULT_EXPIRATION
         self.vrfy_timeout = vrfy_timeout or DEFAULT_VRFY_TIMEOUT
         self.max_request_bytes = max_request_bytes or DEFAULT_MAX_REQ_BYTES
+        self.server_stats = ServerStats(
+            version='python-passcrow v%s' % VERSION,
+            start_ts=time.time(),
+            requests={},
+            storage={})
 
         if not payments:
             payments = [PaymentFree(min(self.expiration, DEFAULT_FREE_TIME))]
@@ -76,43 +93,69 @@ class PasscrowServer:
                         PaymentHashcash(self.storage, bits, self.expiration))
                     break
         self.payments = dict((p.scheme_id, p) for p in payments)
-        self.handlers = handlers or {'mailto': MailtoHandler()}
-        self.warnings_to = warnings_to
+        self.handlers = handlers or {
+            'mailto': EmailHandler(),
+            'email': EmailHandler()}
         self.endpoints = {
+            'stats': self.generate_Stats,
             'policy': self.generate_Policy,
             'escrowrequest': self.process_EscrowRequest,
             'deletionrequest': self.process_DeletionRequest,
             'recoveryrequest': self.process_RecoveryRequest,
             'verificationrequest': self.process_VerificationRequest}
+        for ep in self.endpoints:
+            self.server_stats.requests[ep+'_ok'] = 0
+            self.server_stats.requests[ep+'_ok_usec'] = 0
+            self.server_stats.requests[ep+'_err'] = 0
 
         for table, columns in self.STORAGE_TABLES.items():
             self.storage.prepare_table(table, columns)
 
     def handle(self, user_info, rpc_method, rdata):
         try:
-            if isinstance(rdata, dict):
-                json_data = rdata
-            else:
-                if len(rdata) > self.max_request_bytes:
-                    return JsonError(error='Request too large')
-                json_data = json.loads(rdata or '{}')
-        except:
-            return JsonError(error='Bad request')
-        
-        rl_id = hashlib.md5(bytes(str(user_info), 'utf-8')).hexdigest()
-        try:
-            self.storage.fetch('rlimit', '0-%s' % rl_id)
-            return JsonError(error='Sorry, rate limited.')
-        except KeyError:
-            self.storage.insert('rlimit', b'ping',
-                row_id=rl_id,
-                expiration=int(time.time() + 1))
+            t0 = time.time()
+            try:
+                if isinstance(rdata, dict):
+                    json_data = rdata
+                else:
+                    if len(rdata) > self.max_request_bytes:
+                        return JsonError(error='Request too large')
+                    json_data = json.loads(rdata or '{}')
+            except:
+                raise Exception('Bad request')
 
-        try:
-            self.log('%s method=%s' % (user_info, rpc_method))
-            return self.endpoints[rpc_method](json_data)
-        except KeyError:
-            return JsonError(error=('Unsupported: %s' % rpc_method))
+            rl_id = hashlib.md5(bytes(str(user_info), 'utf-8')).hexdigest()
+            try:
+                self.storage.fetch('rlimit', '0-%s' % rl_id)
+                raise Exception('Sorry, rate limited.')
+            except KeyError:
+                self.storage.insert('rlimit', b'ping',
+                    row_id=rl_id,
+                    expiration=int(time.time() + 1))
+
+            try:
+                self.log('%s method=%s' % (user_info, rpc_method))
+                rv = self.endpoints[rpc_method](json_data)
+
+                # Calculate and update performance stats
+                self.server_stats.requests[rpc_method+'_ok'] += 1
+                rus = int(1000000 * (time.time() - t0))
+                wus = self.server_stats.requests[rpc_method+'_ok_usec']
+                wus = int((0.1 * rus) + (0.9 * wus)) if wus else rus
+                self.server_stats.requests[rpc_method+'_ok_usec'] = wus
+
+                return rv
+            except KeyError:
+                raise Exception(('Unsupported: %s' % rpc_method))
+        except Exception as e:
+            if rpc_method in self.endpoints:
+                ekey += rpc_method + '_err'
+                self.server_stats.requests[rpc_method] += 1
+            return JsonError(error=str(e))
+
+    def generate_Stats(self, request_dict):
+        self.server_stats.storage = self.storage.get_stats()
+        return self.server_stats
 
     def generate_Policy(self, request_dict):
         po = PolicyObject()
@@ -160,11 +203,6 @@ class PasscrowServer:
                 resp.error = 'Insufficient payment'
                 return resp
 
-            # If the user has requested service uptime warnings, add them
-            # to our "mailing list" for such things.
-            if 'warnings-to' in reqp and reqp.warnings_to and self.warnings_to:
-                self.warnings_to(reqp.warnings_to, resp.expiration)
-
             # If we get this far, we know we can decrypt data from this
             # client and verify their Identity. And they've paid!
             resp.escrow_data_id = _id = self.storage.insert('escrow',
@@ -217,15 +255,15 @@ class PasscrowServer:
 
             tmo = esd.timeout if ('timeout' in esd) else self.vrfy_timeout
             tmo = min(tmo, self.vrfy_timeout)
-            vcode = '%s-%6.6d' % (req.prefix, random_int(1000000))
+            vcode = '%s%6.6d' % (req.prefix, random_int(1000000))
             resp.expiration = int(time.time()) + tmo
             self.storage.delete('vcodes', _id)
             self.storage.insert('vcodes', vcode,
                 row_id=_id,
                 expiration=resp.expiration)
 
-            resp.hint = handler.send_code(
-                self, esd.verify, esd.description, vcode, tmo)
+            resp.hint = handler.send_code(self,
+                req.language or 'en', esd.verify, esd.description, vcode, tmo)
 
             return resp
         except KeyboardInterrupt:
@@ -262,20 +300,23 @@ class PasscrowServer:
     @classmethod
     def Init(cls, user, config_file, data_dir, force=False):
         import pwd
-        user_info = pwd.getpwnam(user)
+        if user not in ('', '-', None):
+            user_info = pwd.getpwnam(user)
+        else:
+            user_info = None
         if isinstance(data_dir, str):
             data_dir = bytes(data_dir, 'utf-8')
         if isinstance(config_file, str):
             config_file = bytes(config_file, 'utf-8')
 
         if not force:
-            if not user_info.pw_uid:
+            if user_info and not user_info.pw_uid:
                 raise ValueError('Passcrow server should not run as root')
             if config_file[-3:] not in (b'.py', '.py'):
                 raise ValueError('Config file name should end in .py')
             if os.path.exists(config_file):
                 raise ValueError(
-                    'Cravenly refusing to overwrite existing config') 
+                    'Cravenly refusing to overwrite existing config')
 
         config_file = os.path.normpath(os.path.abspath(config_file))
         for dpath, mode in (
@@ -284,7 +325,8 @@ class PasscrowServer:
             if dpath and not os.path.isdir(dpath):
                 os.mkdir(dpath, mode)
 
-        os.chown(data_dir, user_info.pw_uid, user_info.pw_gid)
+        if user_info:
+            os.chown(data_dir, user_info.pw_uid, user_info.pw_gid)
         with open(config_file, 'w') as fd:
             fd.write("""\
 ## Passcrow Server configuration
@@ -295,7 +337,7 @@ class PasscrowServer:
 ###############################################################################
 
 # This is where the default escrow database is stored.
-data_dir = %s 
+data_dir = %s
 
 
 # Server description.
@@ -324,9 +366,9 @@ vrfy_timeout      = 24 * 3600        # Max time-to-live for verification codes
 
 # Verification handlers
 #
-from passcrow.handlers.email import MailtoHandler
+from passcrow.handlers.email import EmailHandler
 
-mailto_handler = MailtoHandler(
+email_handler = EmailHandler(
 #
 # Uncomment and update SMTP server settings:
 #
@@ -344,14 +386,19 @@ mailto_handler = MailtoHandler(
 
 
 # Use a Twilio account to send SMS messages
-#sms_handler = passcrow.handlers.TwilioSmsHandler(
-#    api_key = '12341241234'),
+#
+#from passcrow.handlers.twilio import TwilioSmsHandler
+#
+#sms_handler = TwilioSmsHandler(
+#    from_number = '+1555123456',
+#    api_sid     = '1234',
+#    api_token   = '12341241234')
 
 
 handlers = {
 #   'sms': sms_handler,        # Uncomment to enable sms: verification
 #   'tel': sms_handler,        # Uncomment to enable tel: verfication
-    'mailto': mailto_handler}
+    'email': email_handler}
 
 #EOF#
 """ % (
@@ -367,7 +414,6 @@ handlers = {
             'log': ValueError,
             'handlers': ValueError,
             'payments': ValueError,
-            'warnings_to': ValueError,
             'country_code': str,
             'about_url': str,
             'expiration': int,
@@ -414,8 +460,14 @@ handlers = {
             **dict((k, config.get(k)) for k in SERVER_SETTINGS))
 
     def cli_cleanup(self):
+        import json
+        stats = {}
         for table in self.STORAGE_TABLES:
-            self.storage.expire_table(table)
+            (expired, live) = self.storage.expire_table(table)
+            stats[table] = {
+                'expired': (expired // len(self.STORAGE_TABLES[table])),
+                'live': (live // len(self.STORAGE_TABLES[table]))}
+        print(json.dumps(stats, indent=2))
         return True
 
 
